@@ -1,8 +1,10 @@
+/* $Cambridge: exim/src/src/lookups/mysql.c,v 1.7 2010/03/05 15:59:29 nm4 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2003 */
+/* Copyright (c) University of Cambridge 1995 - 2009 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Thanks to Paul Kelly for contributing the original code for these
@@ -11,22 +13,6 @@ functions. */
 
 #include "../exim.h"
 #include "lf_functions.h"
-#include "mysql.h"       /* The local header */
-
-
-/* We can't just compile this code and allow the library mechanism to omit the
-functions if they are not wanted, because we need to have the MYSQL header
-available for compiling. Therefore, compile these functions only if
-LOOKUP_MYSQL is defined. However, some compilers don't like compiling empty
-modules, so keep them happy with a dummy when skipping the rest. Make it
-reference itself to stop picky compilers complaining that it is unused, and put
-in a dummy argument to stop even pickier compilers complaining about infinite
-loops. */
-
-#ifndef LOOKUP_MYSQL
-static void dummy(int x) { dummy(x-1); }
-#else
-
 
 #include <mysql.h>       /* The system header */
 
@@ -49,7 +35,7 @@ static mysql_connection *mysql_connections = NULL;
 
 /* See local README for interface description. */
 
-void *
+static void *
 mysql_open(uschar *filename, uschar **errmsg)
 {
 return (void *)(1);    /* Just return something non-null */
@@ -63,7 +49,7 @@ return (void *)(1);    /* Just return something non-null */
 
 /* See local README for interface description. */
 
-void
+static void
 mysql_tidy(void)
 {
 mysql_connection *cn;
@@ -72,9 +58,6 @@ while ((cn = mysql_connections) != NULL)
   mysql_connections = cn->next;
   DEBUG(D_lookup) debug_printf("close MYSQL connection: %s\n", cn->server);
   mysql_close(cn->handle);
-  store_free(cn->server);
-  store_free(cn->handle);
-  store_free(cn);
   }
 }
 
@@ -93,6 +76,7 @@ Arguments:
   resultptr    where to store the result
   errmsg       where to point an error message
   defer_break  TRUE if no more servers are to be tried after DEFER
+  do_cache     set false if data is changed
 
 The server string is of the form "host/dbname/user/password". The host can be
 host:port. This string is in a nextinlist temporary buffer, so can be
@@ -103,7 +87,7 @@ Returns:       OK, FAIL, or DEFER
 
 static int
 perform_mysql_search(uschar *query, uschar *server, uschar **resultptr,
-  uschar **errmsg, BOOL *defer_break)
+  uschar **errmsg, BOOL *defer_break, BOOL *do_cache)
 {
 MYSQL *mysql_handle = NULL;        /* Keep compilers happy */
 MYSQL_RES *mysql_result = NULL;
@@ -137,7 +121,7 @@ for (i = 3; i > 0; i--)
     }
   *pp++ = 0;
   sdata[i] = pp;
-  if (i == 3) server_copy = string_copy_malloc(server);  /* sans password */
+  if (i == 3) server_copy = string_copy(server);  /* sans password */
   }
 sdata[0] = server;   /* What's left at the start */
 
@@ -197,24 +181,22 @@ if (cn == NULL)
 
   /* Get store for a new handle, initialize it, and connect to the server */
 
-  mysql_handle = store_malloc(sizeof(MYSQL));
+  mysql_handle = store_get(sizeof(MYSQL));
   mysql_init(mysql_handle);
   if (mysql_real_connect(mysql_handle,
       /*  host        user         passwd     database */
       CS sdata[0], CS sdata[2], CS sdata[3], CS sdata[1],
-      port, CS socket, 0) == NULL)
+      port, CS socket, CLIENT_MULTI_RESULTS) == NULL)
     {
     *errmsg = string_sprintf("MYSQL connection failed: %s",
       mysql_error(mysql_handle));
-    store_free(mysql_handle);
-    store_free(server_copy);
     *defer_break = FALSE;
     goto MYSQL_EXIT;
     }
 
   /* Add the connection to the cache */
 
-  cn = store_malloc(sizeof(mysql_connection));
+  cn = store_get(sizeof(mysql_connection));
   cn->server = server_copy;
   cn->handle = mysql_handle;
   cn->next = mysql_connections;
@@ -243,7 +225,9 @@ if (mysql_query(mysql_handle, CS query) != 0)
 namely INSERT, UPDATE, or DELETE, an error occurs here. However, this situation
 can be detected by calling mysql_field_count(). If its result is zero, no data
 was expected (this is all explained clearly in the MySQL manual). In this case,
-we return the number of rows affected by the command. */
+we return the number of rows affected by the command. In this event, we do NOT
+want to cache the result; also the whole cache for the handle must be cleaned
+up. Setting do_cache FALSE requests this. */
 
 if ((mysql_result = mysql_use_result(mysql_handle)) == NULL)
   {
@@ -251,6 +235,7 @@ if ((mysql_result = mysql_use_result(mysql_handle)) == NULL)
     {
     DEBUG(D_lookup) debug_printf("MYSQL: query was not one that returns data\n");
     result = string_sprintf("%d", mysql_affected_rows(mysql_handle));
+    *do_cache = FALSE;
     goto MYSQL_EXIT;
     }
   *errmsg = string_sprintf("MYSQL: lookup result failed: %s\n",
@@ -285,39 +270,26 @@ while ((mysql_row_data = mysql_fetch_row(mysql_result)) != NULL)
 
   else for (i = 0; i < num_fields; i++)
     {
-    result = string_cat(result, &ssize, &offset, US fields[i].name,
-      Ustrlen(fields[i].name));
-    result = string_cat(result, &ssize, &offset, US"=", 1);
-
-    /* Quote the value if it contains spaces or is empty */
-
-    if (mysql_row_data[i] == NULL)    /* NULL value */
-      {
-      result = string_cat(result, &ssize, &offset, US"\"\"", 2);
-      }
-
-    else if (mysql_row_data[i][0] == 0 ||
-             Ustrchr(mysql_row_data[i], ' ') != NULL)
-      {
-      int j;
-      result = string_cat(result, &ssize, &offset, US"\"", 1);
-      for (j = 0; j < lengths[i]; j++)
-        {
-	if (mysql_row_data[i][j] == '\"' || mysql_row_data[i][j] == '\\')
-	  result = string_cat(result, &ssize, &offset, US"\\", 1);
-	result = string_cat(result, &ssize, &offset, US mysql_row_data[i]+j, 1);
-        }
-      result = string_cat(result, &ssize, &offset, US"\"", 1);
-      }
-    else
-      {
-      result = string_cat(result, &ssize, &offset, US mysql_row_data[i],
-        lengths[i]);
-      }
-    result = string_cat(result, &ssize, &offset, US" ", 1);
+    result = lf_quote(US fields[i].name, US mysql_row_data[i], lengths[i],
+      result, &ssize, &offset);
     }
   }
 
+/* more results? -1 = no, >0 = error, 0 = yes (keep looping)
+   This is needed because of the CLIENT_MULTI_RESULTS on mysql_real_connect(),
+   we don't expect any more results. */
+
+while((i = mysql_next_result(mysql_handle)) >= 0) {
+   if(i == 0) {   /* Just ignore more results */
+     DEBUG(D_lookup) debug_printf("MYSQL: got unexpected more results\n");
+     continue;
+   }
+
+   *errmsg = string_sprintf("MYSQL: lookup result error when checking for more results: %s\n",
+       mysql_error(mysql_handle));
+   goto MYSQL_EXIT;
+}
+
 /* If result is NULL then no data has been found and so we return FAIL.
 Otherwise, we must terminate the string which has been built; string_cat()
 always leaves enough room for a terminating zero. */
@@ -327,9 +299,14 @@ if (result == NULL)
   yield = FAIL;
   *errmsg = US"MYSQL: no data found";
   }
-else result[offset] = 0;
+else
+  {
+  result[offset] = 0;
+  store_reset(result + offset + 1);
+  }
 
-/* Get here by goto from various error checks. */
+/* Get here by goto from various error checks and from the case where no data
+was read (e.g. an update query). */
 
 MYSQL_EXIT:
 
@@ -340,7 +317,7 @@ if (mysql_result != NULL) mysql_free_result(mysql_result);
 
 /* Non-NULL result indicates a sucessful result */
 
-if(result != NULL)
+if (result != NULL)
   {
   *resultptr = result;
   return OK;
@@ -360,31 +337,16 @@ else
 *************************************************/
 
 /* See local README for interface description. The handle and filename
-arguments are not used. Loop through a list of servers while the query is
-deferred with a retryable error. */
+arguments are not used. The code to loop through a list of servers while the
+query is deferred with a retryable error is now in a separate function that is
+shared with other SQL lookups. */
 
-int
+static int
 mysql_find(void *handle, uschar *filename, uschar *query, int length,
-  uschar **result, uschar **errmsg)
+  uschar **result, uschar **errmsg, BOOL *do_cache)
 {
-int sep = 0;
-uschar *server;
-uschar *list = mysql_servers;
-uschar buffer[512];
-
-DEBUG(D_lookup) debug_printf("MYSQL query: %s\n", query);
-
-while ((server = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
-  {
-  BOOL defer_break = FALSE;
-  int rc = perform_mysql_search(query, server, result, errmsg, &defer_break);
-  if (rc != DEFER || defer_break) return rc;
-  }
-
-if (mysql_servers == NULL)
-  *errmsg = US"no MYSQL servers defined (mysql_servers option)";
-
-return DEFER;
+return lf_sqlperform(US"MySQL", US"mysql_servers", mysql_servers, query,
+  result, errmsg, do_cache, perform_mysql_search);
 }
 
 
@@ -409,7 +371,7 @@ Arguments:
 Returns:     the processed string or NULL for a bad option
 */
 
-uschar *
+static uschar *
 mysql_quote(uschar *s, uschar *opt)
 {
 register int c;
@@ -452,6 +414,45 @@ return quoted;
 }
 
 
-#endif  /* MYSQL_LOOKUP */
+/*************************************************
+*         Version reporting entry point          *
+*************************************************/
+
+/* See local README for interface description. */
+
+#include "../version.h"
+
+void
+mysql_version_report(FILE *f)
+{
+fprintf(f, "Library version: MySQL: Compile: %s [%s]\n"
+           "                        Runtime: %s\n",
+        MYSQL_SERVER_VERSION, MYSQL_COMPILATION_COMMENT,
+        mysql_get_client_info());
+#ifdef DYNLOOKUP
+fprintf(f, "                        Exim version %s\n", EXIM_VERSION_STR);
+#endif
+}
+
+/* These are the lookup_info blocks for this driver */
+
+static lookup_info mysql_lookup_info = {
+  US"mysql",                     /* lookup name */
+  lookup_querystyle,             /* query-style lookup */
+  mysql_open,                    /* open function */
+  NULL,                          /* no check function */
+  mysql_find,                    /* find function */
+  NULL,                          /* no close function */
+  mysql_tidy,                    /* tidy function */
+  mysql_quote,                   /* quoting function */
+  mysql_version_report           /* version reporting */
+};
+
+#ifdef DYNLOOKUP
+#define mysql_lookup_module_info _lookup_module_info
+#endif
+
+static lookup_info *_lookup_list[] = { &mysql_lookup_info };
+lookup_module_info mysql_lookup_module_info = { LOOKUP_MODULE_INFO_MAGIC, _lookup_list, 1 };
 
 /* End of lookups/mysql.c */
